@@ -1,10 +1,11 @@
-from moku import Moku
-from moku.exceptions import MokuException
-from moku.utilities import find_moku_by_serial
-from moku.utilities import validate_range
+import json
+
+from moku import Moku, MultiInstrumentSlottable
+from moku.exceptions import StreamException
+from moku.instruments._stream import StreamInstrument
 
 
-class DigitalFilterBox(Moku):
+class DigitalFilterBox(MultiInstrumentSlottable, Moku, StreamInstrument):
     """
     DigitalFilterBox instrument object.
 
@@ -20,46 +21,77 @@ class DigitalFilterBox(Moku):
 
     """
 
-    def __init__(self, ip=None, serial=None, force_connect=False,
-                 ignore_busy=False, persist_state=False,
-                 connect_timeout=15, read_timeout=30):
-        self.id = 6
-        self.operation_group = "digitalfilterbox"
+    INSTRUMENT_ID = 6
+    OPERATION_GROUP = "digitalfilterbox"
 
-        if not any([ip, serial]):
-            raise MokuException("IP (or) Serial is required")
-        if serial:
-            ip = find_moku_by_serial(serial)
+    def __init__(
+        self,
+        ip=None,
+        serial=None,
+        force_connect=False,
+        ignore_busy=False,
+        persist_state=False,
+        connect_timeout=15,
+        read_timeout=30,
+        slot=None,
+        multi_instrument=None,
+        **kwargs,
+    ):
+        self._init_instrument(
+            ip=ip,
+            serial=serial,
+            force_connect=force_connect,
+            ignore_busy=ignore_busy,
+            persist_state=persist_state,
+            connect_timeout=connect_timeout,
+            read_timeout=read_timeout,
+            slot=slot,
+            multi_instrument=multi_instrument,
+            **kwargs,
+        )
+        StreamInstrument.__init__(self, self.mokuOS_version)
 
-        super().__init__(ip=ip, force_connect=force_connect,
-                         ignore_busy=ignore_busy,
-                         persist_state=persist_state,
-                         connect_timeout=connect_timeout,
-                         read_timeout=read_timeout)
+    @classmethod
+    def for_slot(cls, slot, multi_instrument):
+        """Configures instrument at given slot in multi instrument mode"""
+        return cls(slot=slot, multi_instrument=multi_instrument)
 
-        self.upload_bitstream(self.id)
+    def save_settings(self, filename):
+        """
+        Save instrument settings to a file. The file name should have
+        a `.mokuconf` extension to be compatible with other tools.
+
+        :type filename: FileDescriptorOrPath
+        :param filename: The path to save the `.mokuconf` file to.
+        """
+        self.session.get_file(f"slot{self.slot}/{self.operation_group}", "save_settings", filename)
+    
+    def load_settings(self, filename):
+        """
+        Load a previously saved `.mokuconf` settings file into the instrument.
+        To create a `.mokuconf` file, either use `save_settings` or the desktop app.
+
+        :type filename: FileDescriptorOrPath
+        :param filename: The path to the `.mokuconf` configuration to load
+        """
+        with open(filename, 'rb') as f:
+            self.session.post_file(f"slot{self.slot}/{self.operation_group}", "load_settings", data=f)
 
     def summary(self):
         """
         summary.
         """
         operation = "summary"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_defaults(self):
         """
         set_defaults.
         """
         operation = "set_defaults"
-        return self.session.post(self.operation_group, operation)
+        return self.session.post(f"slot{self.slot}/{self.operation_group}", operation)
 
-    def set_frontend(
-            self,
-            channel,
-            coupling,
-            impedance,
-            attenuation,
-            strict=True):
+    def set_frontend(self, channel, coupling, impedance, attenuation=None, gain=None, strict=True):
         """
         set_frontend.
 
@@ -70,32 +102,32 @@ class DigitalFilterBox(Moku):
         :param channel: Target channel
 
         :type coupling: `string` ['AC', 'DC']
-        :param coupling: Input Coupling
+        :param coupling: Input coupling
 
         :type impedance: `string` ['1MOhm', '50Ohm']
-        :param impedance: Impedance
+        :param impedance: Input impedance
 
-        :type attenuation: `string` ['0dB', '-14dB', '-20dB', '-40dB']
+        :type attenuation: `string` ['-20dB', '0dB', '14dB', '20dB', '32dB', '40dB'] # noqa
         :param attenuation: Input attenuation.
 
+        :type gain: `string` ['20dB', '0dB', '-14dB', '-20dB', '-32dB', '-40dB'] # noqa
+        :param attenuation: Input gain.
+
         """
         operation = "set_frontend"
         params = dict(
-            strict=strict, channel=channel, coupling=validate_range(
-                coupling, [
-                    'AC', 'DC']), impedance=validate_range(
-                impedance, [
-                    '1MOhm', '50Ohm']), attenuation=validate_range(
-                        attenuation, [
-                            '0dB', '-14dB', '-20dB', '-40dB']), )
-        return self.session.post(self.operation_group, operation, params)
+            strict=strict,
+            channel=channel,
+            coupling=coupling,
+            impedance=impedance,
+            attenuation=attenuation,
+            gain=gain,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def set_control_matrix(
-            self,
-            channel,
-            input_gain1,
-            input_gain2,
-            strict=True):
+    def set_control_matrix(self, channel, input_gain1, input_gain2, strict=True):
         """
         set_control_matrix.
 
@@ -119,7 +151,9 @@ class DigitalFilterBox(Moku):
             input_gain1=input_gain1,
             input_gain2=input_gain2,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_monitor(self, monitor_channel, source, strict=True):
         """
@@ -131,37 +165,23 @@ class DigitalFilterBox(Moku):
         :type monitor_channel: `integer`
         :param monitor_channel: Monitor channel
 
-        :type source: `string` ['None', 'Input1', 'Filter1', 'Output1', 'Input2', 'Filter2', 'Output2', 'Input3', 'Filter3', 'Output3', 'Input4', 'Filter4', 'Output4']
+        :type source: `string` ['None', 'Input1', 'Filter1', 'Output1', 'Input2', 'Filter2', 'Output2', 'Input3', 'Filter3', 'Output3', 'Input4', 'Filter4', 'Output4'] # noqa
         :param source: Monitor channel source.
 
         """
         operation = "set_monitor"
-        params = dict(strict=strict,
-                      monitor_channel=monitor_channel,
-                      source=validate_range(source,
-                                            ['None',
-                                             'Input1',
-                                             'Filter1',
-                                             'Output1',
-                                             'Input2',
-                                             'Filter2',
-                                             'Output2',
-                                             'Input3',
-                                             'Filter3',
-                                             'Output3',
-                                             'Input4',
-                                             'Filter4',
-                                             'Output4']),
-                      )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            monitor_channel=monitor_channel,
+            source=source,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def enable_output(
-            self,
-            channel,
-            signal=True,
-            output=True,
-            gain_range="0dB",
-            strict=True):
+        self, channel, signal=True, output=True, gain_range="0dB", strict=True
+    ):
         """
         enable_output.
 
@@ -189,7 +209,9 @@ class DigitalFilterBox(Moku):
             output=output,
             gain_range=gain_range,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_input_offset(self, channel, offset, strict=True):
         """
@@ -211,7 +233,9 @@ class DigitalFilterBox(Moku):
             channel=channel,
             offset=offset,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_output_offset(self, channel, offset, strict=True):
         """
@@ -233,7 +257,9 @@ class DigitalFilterBox(Moku):
             channel=channel,
             offset=offset,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_input_gain(self, channel, gain, strict=True):
         """
@@ -255,7 +281,9 @@ class DigitalFilterBox(Moku):
             channel=channel,
             gain=gain,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_output_gain(self, channel, gain, strict=True):
         """
@@ -277,24 +305,28 @@ class DigitalFilterBox(Moku):
             channel=channel,
             gain=gain,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_trigger(
-            self,
-            type="Edge",
-            level=0,
-            mode="Auto",
-            edge="Rising",
-            polarity="Positive",
-            width=0.0001,
-            width_condition="LessThan",
-            nth_event=1,
-            holdoff=0,
-            auto_sensitivity=True,
-            noise_reject=False,
-            hf_reject=False,
-            source="ProbeA",
-            strict=True):
+        self,
+        type="Edge",
+        level=0,
+        mode="Auto",
+        edge="Rising",
+        polarity="Positive",
+        width=0.0001,
+        width_condition="LessThan",
+        nth_event=1,
+        holdoff=0,
+        hysteresis=1e-3,
+        auto_sensitivity=None,
+        noise_reject=False,
+        hf_reject=False,
+        source="ProbeA",
+        strict=True,
+    ):
         """
         set_trigger.
 
@@ -311,7 +343,7 @@ class DigitalFilterBox(Moku):
         :param mode: Trigger mode
 
         :type edge: `string` ['Rising', 'Falling', 'Both']
-        :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising) or negative (falling), with the 'both' option being invalid
+        :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising) or negative (falling), with the 'both' option being invalid # noqa
 
         :type polarity: `string` ['Positive', 'Negative']
         :param polarity: Trigger pulse polarity (Pulse mode only)
@@ -323,55 +355,62 @@ class DigitalFilterBox(Moku):
         :param width_condition: Trigger pulse width condition (pulse mode only)
 
         :type nth_event: `integer` [0, 65535]  (defaults to 1)
-        :param nth_event: The number of trigger events to wait for before triggering
+        :param nth_event: The number of trigger events to wait for before triggering # noqa
 
         :type holdoff: `number` [1e-9Sec, 10Sec]  (defaults to 0)
-        :param holdoff: The duration to hold-off Oscilloscope trigger post trigger event
+        :param holdoff: The duration to hold-off Oscilloscope trigger post trigger event # noqa
+
+        :type hysteresis: `number` (defaults to 0.1)
+        :param hysteresis: Absolute hysteresis around trigger
 
         :type auto_sensitivity: `boolean`
         :param auto_sensitivity: Configure auto or manual hysteresis for noise rejection.
 
         :type noise_reject: `boolean`
-        :param noise_reject: Configure the Oscilloscope with a small amount of hysteresis to prevent repeated triggering due to noise
+        :param noise_reject: Configure the Oscilloscope with a small amount of hysteresis to prevent repeated triggering due to noise # noqa
 
         :type hf_reject: `boolean`
-        :param hf_reject: Configure the trigger signal to pass through a low pass filter to smooths out the noise
+        :param hf_reject: Configure the trigger signal to pass through a low pass filter to smooths out the noise # noqa
 
-        :type source: `string` ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD']
+        :type source: `string` ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External']
         :param source: Trigger Source
 
         """
         operation = "set_trigger"
         params = dict(
             strict=strict,
-            type=validate_range(type, ['Edge', 'Pulse']),
+            type=type,
             level=level,
-            mode=validate_range(mode, ['Auto', 'Normal']),
-            edge=validate_range(edge, ['Rising', 'Falling', 'Both']),
-            polarity=validate_range(polarity, ['Positive', 'Negative']),
+            mode=mode,
+            edge=edge,
+            polarity=polarity,
             width=width,
-            width_condition=validate_range(width_condition, ['GreaterThan', 'LessThan']),
+            width_condition=width_condition,
             nth_event=nth_event,
             holdoff=holdoff,
+            hysteresis=hysteresis,
             auto_sensitivity=auto_sensitivity,
             noise_reject=noise_reject,
             hf_reject=hf_reject,
-            source=validate_range(source, ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD']),
+            source=source,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
     def set_filter(
-            self,
-            channel,
-            sample_rate,
-            shape="Lowpass",
-            type="Butterworth",
-            low_corner=None,
-            high_corner=None,
-            pass_band_ripple=None,
-            stop_band_attenuation=None,
-            order=8,
-            strict=True):
+        self,
+        channel,
+        sample_rate,
+        shape="Lowpass",
+        type="Butterworth",
+        low_corner=None,
+        high_corner=None,
+        pass_band_ripple=None,
+        stop_band_attenuation=None,
+        order=8,
+        strict=True,
+    ):
         """
         set_filter.
 
@@ -381,13 +420,13 @@ class DigitalFilterBox(Moku):
         :type channel: `integer`
         :param channel: Target channel
 
-        :type sample_rate: `string` ['3.906MHz', '488.3kHz', '61.04kHz', '39.06MHz', '4.883MHz', '305.2kHz']
+        :type sample_rate: `string` ['3.906MHz', '488.3kHz', '61.04kHz', '39.06MHz', '4.883MHz', '305.2kHz', '15.625MHz', '1.9531MHz', '122.07kHz'] # noqa
         :param sample_rate: Sample rate
 
         :type shape: `string` ['Lowpass', 'Highpass', 'Bandpass', 'Bandstop']
         :param shape: Filter shape
 
-        :type type: `string` ['Butterworth', 'ChebyshevI', 'ChebyshevII', 'Elliptic', 'Cascaded', 'Bessel', 'Gaussian', 'Legendre']
+        :type type: `string` ['Butterworth', 'ChebyshevI', 'ChebyshevII', 'Elliptic', 'Cascaded', 'Bessel', 'Gaussian', 'Legendre'] # noqa
         :param type: Filter type
 
         :type low_corner: `number`
@@ -407,44 +446,25 @@ class DigitalFilterBox(Moku):
 
         """
         operation = "set_filter"
-        params = dict(strict=strict,
-                      channel=channel,
-                      sample_rate=validate_range(sample_rate,
-                                                 ['3.906MHz',
-                                                  '488.3kHz',
-                                                  '61.04kHz',
-                                                  '39.06MHz',
-                                                  '4.883MHz',
-                                                  '305.2kHz']),
-                      shape=validate_range(shape,
-                                           ['Lowpass',
-                                            'Highpass',
-                                            'Bandpass',
-                                            'Bandstop']),
-                      type=validate_range(type,
-                                          ['Butterworth',
-                                           'ChebyshevI',
-                                           'ChebyshevII',
-                                           'Elliptic',
-                                           'Cascaded',
-                                           'Bessel',
-                                           'Gaussian',
-                                           'Legendre']),
-                      low_corner=low_corner,
-                      high_corner=high_corner,
-                      pass_band_ripple=pass_band_ripple,
-                      stop_band_attenuation=stop_band_attenuation,
-                      order=order,
-                      )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            channel=channel,
+            sample_rate=sample_rate,
+            shape=shape,
+            type=type,
+            low_corner=low_corner,
+            high_corner=high_corner,
+            pass_band_ripple=pass_band_ripple,
+            stop_band_attenuation=stop_band_attenuation,
+            order=order,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_custom_filter(
-            self,
-            channel,
-            sample_rate,
-            scaling=1,
-            coefficients=None,
-            strict=True):
+        self, channel, sample_rate, scaling=1, coefficients=None, strict=True
+    ):
         """
         set_custom_filter.
 
@@ -454,138 +474,123 @@ class DigitalFilterBox(Moku):
         :type channel: `integer`
         :param channel: Target channel
 
-        :type sample_rate: `string` ['3.906MHz', '488.3kHz', '61.04kHz', '39.06MHz', '4.883MHz', '305.2kHz']
+        :type sample_rate: `string` ['3.906MHz', '488.3kHz', '61.04kHz', '39.06MHz', '4.883MHz', '305.2kHz', '15.625MHz', '1.9531MHz', '122.07kHz'] # noqa
         :param sample_rate: Sample rate
 
         :type scaling: `number`
         :param scaling: Output scaling
 
         :type coefficients: `list`
-        :param coefficients: List of filter stages, where each stage should have six coefficients and each coefficient must be in the range [-4.0, 4.0]
+        :param coefficients: List of filter stages, where each stage should have six coefficients and each coefficient must be in the range [-4.0, 4.0] # noqa
 
         """
         operation = "set_custom_filter"
-        params = dict(strict=strict,
-                      channel=channel,
-                      sample_rate=validate_range(sample_rate,
-                                                 ['3.906MHz',
-                                                  '488.3kHz',
-                                                  '61.04kHz',
-                                                  '39.06MHz',
-                                                  '4.883MHz',
-                                                  '305.2kHz']),
-                      scaling=scaling,
-                      coefficients=coefficients,
-                      )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            channel=channel,
+            sample_rate=sample_rate,
+            scaling=scaling,
+            coefficients=coefficients,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_frontend(self, channel, strict=True):
+    def get_frontend(self, channel):
         """
         get_frontend.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_frontend"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_control_matrix(self, channel, strict=True):
+    def get_control_matrix(self, channel):
         """
         get_control_matrix.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_control_matrix"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_input_offset(self, channel, strict=True):
+    def get_input_offset(self, channel):
         """
         get_input_offset.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_input_offset"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_output_offset(self, channel, strict=True):
+    def get_output_offset(self, channel):
         """
         get_output_offset.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_output_offset"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_input_gain(self, channel, strict=True):
+    def get_input_gain(self, channel):
         """
         get_input_gain.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_input_gain"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_output_gain(self, channel, strict=True):
+    def get_output_gain(self, channel):
         """
         get_output_gain.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_output_gain"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_timebase(self, t1, t2, strict=True):
         """
@@ -598,7 +603,7 @@ class DigitalFilterBox(Moku):
         :param t1: Time from the trigger point to the left of screen. This may be negative (trigger on-screen) or positive (trigger off the left of screen).
 
         :type t2: `number`
-        :param t2: Time from the trigger point to the right of screen. (Must be a positive number, i.e. after the trigger event)
+        :param t2: Time from the trigger point to the right of screen. (Must be a positive number, i.e. after the trigger event) # noqa
 
         """
         operation = "set_timebase"
@@ -607,12 +612,17 @@ class DigitalFilterBox(Moku):
             t1=t1,
             t2=t2,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_hysteresis(self, hysteresis_mode, value=0, strict=True):
         """
         set_hysteresis.
 
+        .. deprecated:: 3.1.1
+        Use `hysteresis` parameter of `set_trigger` instead.
+
         :type strict: `boolean`
         :param strict: Disable all implicit conversions and coercions.
 
@@ -624,9 +634,14 @@ class DigitalFilterBox(Moku):
 
         """
         operation = "set_hysteresis"
-        params = dict(strict=strict, hysteresis_mode=validate_range(
-            hysteresis_mode, ['Absolute', 'Relative']), value=value, )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            hysteresis_mode=hysteresis_mode,
+            value=value,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def enable_rollmode(self, roll=True, strict=True):
         """
@@ -644,9 +659,13 @@ class DigitalFilterBox(Moku):
             strict=strict,
             roll=roll,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_data(self, timeout=60, wait_reacquire=False):
+    def get_data(
+        self, timeout=60, wait_reacquire=False, wait_complete=False, measurements=False
+    ):
         """
         get_data.
 
@@ -656,6 +675,11 @@ class DigitalFilterBox(Moku):
         :type wait_reacquire: `boolean`
         :param wait_reacquire: Wait until new dataframe is reacquired
 
+        :type wait_complete: `boolean`
+        :param wait_complete: Wait until entire frame is available
+
+        :type measurements: `boolean`
+        :param measurements: When True, includes available measurements for each channel
 
         .. important::
             Default timeout for reading the data is 10 seconds. It
@@ -669,29 +693,29 @@ class DigitalFilterBox(Moku):
         params = dict(
             timeout=timeout,
             wait_reacquire=wait_reacquire,
+            wait_complete=wait_complete,
+            measurements=measurements,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
-    def save_high_res_buffer(self, comments=""):
+    def save_high_res_buffer(self, comments="", timeout=60):
         """
         save_high_res_buffer.
 
         :type comments: `string`
         :param comments: Optional comments to be included
 
+        :type timeout: `int`
+        :param timeout: Wait for n seconds before trigger event happens to save the buffer # noqa
+
         """
         operation = "save_high_res_buffer"
-        params = dict(
-            comments=comments,
+        params = dict(comments=comments, timeout=timeout)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
-
-    def get_samplerate(self):
-        """
-        get_samplerate.
-        """
-        operation = "get_samplerate"
-        return self.session.get(self.operation_group, operation)
 
     def set_acquisition_mode(self, mode="Normal", strict=True):
         """
@@ -700,45 +724,56 @@ class DigitalFilterBox(Moku):
         :type strict: `boolean`
         :param strict: Disable all implicit conversions and coercions.
 
-        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
         :param mode: Acquisition Mode
 
         """
         operation = "set_acquisition_mode"
-        params = dict(strict=strict, mode=validate_range(
-            mode, ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']), )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(strict=strict, mode=mode)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+
+    def get_samplerate(self):
+        """
+        get_samplerate.
+        """
+        operation = "get_samplerate"
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def get_acquisition_mode(self):
         """
         get_acquisition_mode.
         """
         operation = "get_acquisition_mode"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def get_timebase(self):
         """
         get_timebase.
         """
         operation = "get_timebase"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def logging_progress(self):
         """
         logging_progress.
         """
         operation = "logging_progress"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def start_logging(
-            self,
-            duration=60,
-            delay=0,
-            file_name_prefix="",
-            comments="",
-            mode="Normal",
-            rate=1000,
-            strict=True):
+        self,
+        duration=60,
+        delay=0,
+        file_name_prefix="",
+        comments="",
+        trigger_source=None,
+        trigger_level=None,
+        mode="Normal",
+        rate=None,
+        strict=True,
+    ):
         """
         start_logging.
 
@@ -751,13 +786,19 @@ class DigitalFilterBox(Moku):
         :type delay: `integer` Sec (defaults to 0)
         :param delay: Delay the start by
 
+        :type trigger_source: `string`  ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External'] # noqa
+        :param trigger_source: Trigger source
+
+        :type trigger_level: `number` [-5V, 5V]  (defaults to 0)
+        :param trigger_level: Trigger level
+
         :type file_name_prefix: `string`
         :param file_name_prefix: Optional file name prefix
 
         :type comments: `string`
         :param comments: Optional comments to be included
 
-        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
         :param mode: Acquisition Mode
 
         :type rate: `number`
@@ -769,16 +810,108 @@ class DigitalFilterBox(Moku):
             strict=strict,
             duration=duration,
             delay=delay,
+            trigger_source=trigger_source,
+            trigger_level=trigger_level,
             file_name_prefix=file_name_prefix,
             comments=comments,
-            mode=validate_range(mode, ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']),
+            mode=mode,
             rate=rate,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def stop_logging(self):
         """
         stop_logging.
         """
         operation = "stop_logging"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
+
+    def start_streaming(
+        self,
+        duration=None,
+        mode="Normal",
+        rate=None,
+        trigger_source=None,
+        trigger_level=None,
+    ):
+        """
+        start_streaming.
+
+        :type duration: `integer`
+        :param duration: Duration in second(s) to stream for
+
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
+        :param mode: Acquisition Mode
+
+        :type rate: `number`
+        :param rate: Acquisition rate
+
+        :type trigger_source: `string`  ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External'] # noqa
+        :param trigger_source: Trigger source
+
+        :type trigger_level: `number` [-5V, 5V]  (defaults to 0)
+        :param trigger_level: Trigger level
+
+
+        """
+        super().start_streaming()
+        operation = "start_streaming"
+        params = dict(
+            duration=duration,
+            mode=mode,
+            rate=rate,
+            trigger_source=trigger_source,
+            trigger_level=trigger_level,
+        )
+
+        response = self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+        self.stream_id = response["stream_id"]
+        self.ip_address = self.session.ip_address
+
+        return response
+
+    def stop_streaming(self):
+        """
+        stop_streaming.
+
+        """
+        operation = "stop_streaming"
+        response = self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation
+        )
+        self.stream_id = None
+        return response
+
+    def get_chunk(self):
+        """
+        get_chunk.
+
+        Get the next raw chunk from the streaming session
+
+        """
+        data = {"stream_id": {self.stream_id: {"topic": f"logformat{self.slot-1}"}}}
+        result = self.session.post_to_v2_raw("get_chunk", params=data)
+
+        if result.status_code != 200:
+            raise StreamException("Error fetching stream.")
+
+        try:
+            error = json.loads(result.content)
+            raise StreamException(error.get("error", error))
+        except StreamException as e:
+            raise e
+        except Exception:
+            return result.content
+
+    def get_stream_status(self):
+        """
+        get_stream_status.
+
+        """
+        operation = "get_stream_status"
+
+        return self.session.post(f"slot{self.slot}/{self.operation_group}", operation)
