@@ -1,10 +1,11 @@
-from moku import Moku
-from moku.exceptions import MokuException
-from moku.utilities import find_moku_by_serial
-from moku.utilities import validate_range
+import json
+
+from moku import Moku, MultiInstrumentSlottable
+from moku.exceptions import StreamException
+from moku.instruments._stream import StreamInstrument
 
 
-class LockInAmp(Moku):
+class LockInAmp(MultiInstrumentSlottable, Moku, StreamInstrument):
     """
     LockInAmp instrument object.
 
@@ -16,46 +17,77 @@ class LockInAmp(Moku):
 
     """
 
-    def __init__(self, ip=None, serial=None, force_connect=False,
-                 ignore_busy=False, persist_state=False,
-                 connect_timeout=15, read_timeout=30):
-        self.id = 8
-        self.operation_group = "lockinamp"
+    INSTRUMENT_ID = 8
+    OPERATION_GROUP = "lockinamp"
 
-        if not any([ip, serial]):
-            raise MokuException("IP (or) Serial is required")
-        if serial:
-            ip = find_moku_by_serial(serial)
+    def __init__(
+        self,
+        ip=None,
+        serial=None,
+        force_connect=False,
+        ignore_busy=False,
+        persist_state=False,
+        connect_timeout=15,
+        read_timeout=30,
+        slot=None,
+        multi_instrument=None,
+        **kwargs,
+    ):
+        self._init_instrument(
+            ip=ip,
+            serial=serial,
+            force_connect=force_connect,
+            ignore_busy=ignore_busy,
+            persist_state=persist_state,
+            connect_timeout=connect_timeout,
+            read_timeout=read_timeout,
+            slot=slot,
+            multi_instrument=multi_instrument,
+            **kwargs,
+        )
+        StreamInstrument.__init__(self, self.mokuOS_version)
 
-        super().__init__(ip=ip, force_connect=force_connect,
-                         ignore_busy=ignore_busy,
-                         persist_state=persist_state,
-                         connect_timeout=connect_timeout,
-                         read_timeout=read_timeout)
+    @classmethod
+    def for_slot(cls, slot, multi_instrument):
+        """Configures instrument at given slot in multi instrument mode"""
+        return cls(slot=slot, multi_instrument=multi_instrument)
 
-        self.upload_bitstream(self.id)
+    def save_settings(self, filename):
+        """
+        Save instrument settings to a file. The file name should have
+        a `.mokuconf` extension to be compatible with other tools.
+
+        :type filename: FileDescriptorOrPath
+        :param filename: The path to save the `.mokuconf` file to.
+        """
+        self.session.get_file(f"slot{self.slot}/{self.operation_group}", "save_settings", filename)
+    
+    def load_settings(self, filename):
+        """
+        Load a previously saved `.mokuconf` settings file into the instrument.
+        To create a `.mokuconf` file, either use `save_settings` or the desktop app.
+
+        :type filename: FileDescriptorOrPath
+        :param filename: The path to the `.mokuconf` configuration to load
+        """
+        with open(filename, 'rb') as f:
+            self.session.post_file(f"slot{self.slot}/{self.operation_group}", "load_settings", data=f)
 
     def summary(self):
         """
         summary.
         """
         operation = "summary"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_defaults(self):
         """
         set_defaults.
         """
         operation = "set_defaults"
-        return self.session.post(self.operation_group, operation)
+        return self.session.post(f"slot{self.slot}/{self.operation_group}", operation)
 
-    def set_frontend(
-            self,
-            channel,
-            coupling,
-            impedance,
-            attenuation,
-            strict=True):
+    def set_frontend(self, channel, coupling, impedance, attenuation=None, gain=None, strict=True):
         """
         set_frontend.
 
@@ -66,43 +98,46 @@ class LockInAmp(Moku):
         :param channel: Target channel
 
         :type coupling: `string` ['AC', 'DC']
-        :param coupling: Input Coupling
+        :param coupling: Input coupling
 
         :type impedance: `string` ['1MOhm', '50Ohm']
-        :param impedance: Impedance
+        :param impedance: Input impedance
 
-        :type attenuation: `string` ['0dB', '-14dB', '-20dB', '-40dB']
+        :type attenuation: `string` ['-20dB', '0dB', '14dB', '20dB', '32dB', '40dB'] # noqa
         :param attenuation: Input attenuation.
 
+        :type gain: `string` ['20dB', '0dB', '-14dB', '-20dB', '-32dB', '-40dB'] # noqa
+        :param attenuation: Input gain.
+
         """
         operation = "set_frontend"
         params = dict(
-            strict=strict, channel=channel, coupling=validate_range(
-                coupling, [
-                    'AC', 'DC']), impedance=validate_range(
-                impedance, [
-                    '1MOhm', '50Ohm']), attenuation=validate_range(
-                        attenuation, [
-                            '0dB', '-14dB', '-20dB', '-40dB']), )
-        return self.session.post(self.operation_group, operation, params)
+            strict=strict,
+            channel=channel,
+            coupling=coupling,
+            impedance=impedance,
+            attenuation=attenuation,
+            gain=gain,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_frontend(self, channel, strict=True):
+    def get_frontend(self, channel):
         """
         get_frontend.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_frontend"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_demodulation(self, mode, frequency=1000000, phase=0, strict=True):
         """
@@ -124,23 +159,28 @@ class LockInAmp(Moku):
         operation = "set_demodulation"
         params = dict(
             strict=strict,
-            mode=validate_range(mode, ['Internal', 'External', 'ExternalPLL', 'None']),
+            mode=mode,
             frequency=frequency,
             phase=phase,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def get_demodulation(self):
         """
         get_demodulation.
         """
         operation = "get_demodulation"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
-    def set_filter(self, corner_frequency, slope="Slope6dB"):
+    def set_filter(self, corner_frequency, slope="Slope6dB", strict=True):
         """
         set_filter.
 
+        :type strict: `boolean`
+        :param strict: Disable all implicit conversions and coercions.
+
         :type corner_frequency: `number`
         :param corner_frequency: Corner frequency of low-pass filter
 
@@ -150,17 +190,20 @@ class LockInAmp(Moku):
         """
         operation = "set_filter"
         params = dict(
+            strict=strict,
             corner_frequency=corner_frequency,
             slope=slope,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def get_filter(self):
         """
         get_filter.
         """
         operation = "get_filter"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_monitor(self, monitor_channel, source, strict=True):
         """
@@ -172,31 +215,26 @@ class LockInAmp(Moku):
         :type monitor_channel: `integer`
         :param monitor_channel: Monitor channel
 
-        :type source: `string` ['None', 'Input1', 'ISignal', 'QSignal', 'MainOutput', 'AuxOutput', 'Input2', 'Demod']
+        :type source: `string` ['None', 'Input1', 'ISignal', 'QSignal', 'MainOutput', 'AuxOutput', 'Input2', 'Demod'] # noqa
         :param source: Monitor channel source.
 
         """
         operation = "set_monitor"
-        params = dict(strict=strict,
-                      monitor_channel=monitor_channel,
-                      source=validate_range(source,
-                                            ['None',
-                                             'Input1',
-                                             'ISignal',
-                                             'QSignal',
-                                             'MainOutput',
-                                             'AuxOutput',
-                                             'Input2',
-                                             'Demod']),
-                      )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            monitor_channel=monitor_channel,
+            source=source,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def get_outputs(self):
         """
         get_outputs.
         """
         operation = "get_outputs"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_outputs(self, main, aux, main_offset=0, aux_offset=0, strict=True):
         """
@@ -221,12 +259,14 @@ class LockInAmp(Moku):
         operation = "set_outputs"
         params = dict(
             strict=strict,
-            main=validate_range(main, ['X', 'Y', 'R', 'Theta', 'Offset', 'None']),
-            aux=validate_range(aux, ['Y', 'Theta', 'Demod', 'Aux', 'Offset', 'None']),
+            main=main,
+            aux=aux,
             main_offset=main_offset,
             aux_offset=aux_offset,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_aux_output(self, frequency, amplitude, strict=True):
         """
@@ -248,24 +288,27 @@ class LockInAmp(Moku):
             frequency=frequency,
             amplitude=amplitude,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def get_aux_output(self):
         """
         get_aux_output.
         """
         operation = "get_aux_output"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_gain(
-            self,
-            main,
-            aux,
-            main_invert=False,
-            aux_invert=False,
-            main_gain_range="0dB",
-            aux_gain_range="0dB",
-            strict=True):
+        self,
+        main,
+        aux,
+        main_invert=False,
+        aux_invert=False,
+        main_gain_range="0dB",
+        aux_gain_range="0dB",
+        strict=True,
+    ):
         """
         set_gain.
 
@@ -301,14 +344,16 @@ class LockInAmp(Moku):
             main_gain_range=main_gain_range,
             aux_gain_range=aux_gain_range,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def get_gain(self):
         """
         get_gain.
         """
         operation = "get_gain"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_polar_mode(self, range, strict=True):
         """
@@ -324,23 +369,27 @@ class LockInAmp(Moku):
         operation = "set_polar_mode"
         params = dict(
             strict=strict,
-            range=validate_range(range, ['2Vpp', '7.5mVpp', '25uVpp']),
+            range=range,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
     def get_polar_theta_range(self):
         """
         get_polar_theta_range.
         """
         operation = "get_polar_theta_range"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_pll(
-            self,
-            auto_acquire=True,
-            frequency=None,
-            bandwidth="1kHz",
-            strict=True):
+        self,
+        auto_acquire=True,
+        frequency=None,
+        frequency_multiplier=1,
+        bandwidth="1kHz",
+        strict=True,
+    ):
         """
         set_pll.
 
@@ -353,31 +402,31 @@ class LockInAmp(Moku):
         :type frequency: `number`
         :param frequency: External PLL frequency(when auto_acquire is false)
 
-        :type bandwidth: `string` ['1Hz', '10Hz', '100Hz', '1kHz', '10kHz', '100kHz', '1MHz']
+        :type frequency_multiplier: `number`
+        :param frequency_multiplier: Frequency multiplier
+
+        :type bandwidth: `string` ['1Hz', '10Hz', '100Hz', '1kHz', '10kHz', '100kHz', '1MHz'] # noqa
         :param bandwidth: Bandwidth.
 
         """
         operation = "set_pll"
-        params = dict(strict=strict,
-                      auto_acquire=auto_acquire,
-                      frequency=frequency,
-                      bandwidth=validate_range(bandwidth,
-                                               ['1Hz',
-                                                '10Hz',
-                                                '100Hz',
-                                                '1kHz',
-                                                '10kHz',
-                                                '100kHz',
-                                                '1MHz']),
-                      )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            auto_acquire=auto_acquire,
+            frequency=frequency,
+            frequency_multiplier=frequency_multiplier,
+            bandwidth=bandwidth,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def get_pll(self):
         """
         get_pll.
         """
         operation = "get_pll"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def use_pid(self, channel, strict=True):
         """
@@ -393,18 +442,22 @@ class LockInAmp(Moku):
         operation = "use_pid"
         params = dict(
             strict=strict,
-            channel=validate_range(channel, ['Off', 'Main', 'Aux']),
+            channel=channel,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
     def set_by_frequency(
-            self,
-            prop_gain=None,
-            int_crossover=None,
-            diff_crossover=None,
-            int_saturation=None,
-            diff_saturation=None,
-            strict=True):
+        self,
+        prop_gain=None,
+        int_crossover=None,
+        diff_crossover=None,
+        int_saturation=None,
+        diff_saturation=None,
+        invert=False,
+        strict=True,
+    ):
         """
         set_by_frequency.
 
@@ -426,6 +479,9 @@ class LockInAmp(Moku):
         :type diff_saturation: `number` [-60dB, 60dB]
         :param diff_saturation: Differentiator gain saturation
 
+        :type invert: `boolean`
+        :param invert: Invert PID
+
         """
         operation = "set_by_frequency"
         params = dict(
@@ -435,32 +491,37 @@ class LockInAmp(Moku):
             diff_crossover=diff_crossover,
             int_saturation=int_saturation,
             diff_saturation=diff_saturation,
+            invert=invert,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
     def pll_reacquire(self):
         """
         pll_reacquire.
         """
         operation = "pll_reacquire"
-        return self.session.post(self.operation_group, operation)
+        return self.session.post(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_trigger(
-            self,
-            type="Edge",
-            level=0,
-            mode="Auto",
-            edge="Rising",
-            polarity="Positive",
-            width=0.0001,
-            width_condition="LessThan",
-            nth_event=1,
-            holdoff=0,
-            auto_sensitivity=True,
-            noise_reject=False,
-            hf_reject=False,
-            source="ProbeA",
-            strict=True):
+        self,
+        type="Edge",
+        level=0,
+        mode="Auto",
+        edge="Rising",
+        polarity="Positive",
+        width=0.0001,
+        width_condition="LessThan",
+        nth_event=1,
+        holdoff=0,
+        hysteresis=1e-3,
+        auto_sensitivity=None,
+        noise_reject=False,
+        hf_reject=False,
+        source="ProbeA",
+        strict=True,
+    ):
         """
         set_trigger.
 
@@ -477,7 +538,7 @@ class LockInAmp(Moku):
         :param mode: Trigger mode
 
         :type edge: `string` ['Rising', 'Falling', 'Both']
-        :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising) or negative (falling), with the 'both' option being invalid
+        :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising) or negative (falling), with the 'both' option being invalid # noqa
 
         :type polarity: `string` ['Positive', 'Negative']
         :param polarity: Trigger pulse polarity (Pulse mode only)
@@ -489,42 +550,48 @@ class LockInAmp(Moku):
         :param width_condition: Trigger pulse width condition (pulse mode only)
 
         :type nth_event: `integer` [0, 65535]  (defaults to 1)
-        :param nth_event: The number of trigger events to wait for before triggering
+        :param nth_event: The number of trigger events to wait for before triggering # noqa
 
         :type holdoff: `number` [1e-9Sec, 10Sec]  (defaults to 0)
-        :param holdoff: The duration to hold-off Oscilloscope trigger post trigger event
+        :param holdoff: The duration to hold-off Oscilloscope trigger post trigger event # noqa
+
+        :type hysteresis: `number` (defaults to 0.1)
+        :param hysteresis: Absolute hysteresis around trigger
 
         :type auto_sensitivity: `boolean`
         :param auto_sensitivity: Configure auto or manual hysteresis for noise rejection.
 
         :type noise_reject: `boolean`
-        :param noise_reject: Configure the Oscilloscope with a small amount of hysteresis to prevent repeated triggering due to noise
+        :param noise_reject: Configure the Oscilloscope with a small amount of hysteresis to prevent repeated triggering due to noise # noqa
 
         :type hf_reject: `boolean`
-        :param hf_reject: Configure the trigger signal to pass through a low pass filter to smooths out the noise
+        :param hf_reject: Configure the trigger signal to pass through a low pass filter to smooths out the noise # noqa
 
-        :type source: `string` ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD']
+        :type source: `string` ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External']
         :param source: Trigger Source
 
         """
         operation = "set_trigger"
         params = dict(
             strict=strict,
-            type=validate_range(type, ['Edge', 'Pulse']),
+            type=type,
             level=level,
-            mode=validate_range(mode, ['Auto', 'Normal']),
-            edge=validate_range(edge, ['Rising', 'Falling', 'Both']),
-            polarity=validate_range(polarity, ['Positive', 'Negative']),
+            mode=mode,
+            edge=edge,
+            polarity=polarity,
             width=width,
-            width_condition=validate_range(width_condition, ['GreaterThan', 'LessThan']),
+            width_condition=width_condition,
             nth_event=nth_event,
             holdoff=holdoff,
+            hysteresis=hysteresis,
             auto_sensitivity=auto_sensitivity,
             noise_reject=noise_reject,
             hf_reject=hf_reject,
-            source=validate_range(source, ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD']),
+            source=source,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
     def set_timebase(self, t1, t2, strict=True):
         """
@@ -534,10 +601,10 @@ class LockInAmp(Moku):
         :param strict: Disable all implicit conversions and coercions.
 
         :type t1: `number`
-        :param t1: Time from the trigger point to the left of screen. This may be negative (trigger on-screen) or positive (trigger off the left of screen).
+        :param t1: Time from the trigger point to the left of screen. This may be negative (trigger on-screen) or positive (trigger off the left of screen). # noqa
 
         :type t2: `number`
-        :param t2: Time from the trigger point to the right of screen. (Must be a positive number, i.e. after the trigger event)
+        :param t2: Time from the trigger point to the right of screen. (Must be a positive number, i.e. after the trigger event) # noqa
 
         """
         operation = "set_timebase"
@@ -546,12 +613,17 @@ class LockInAmp(Moku):
             t1=t1,
             t2=t2,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_hysteresis(self, hysteresis_mode, value=0, strict=True):
         """
         set_hysteresis.
 
+        .. deprecated:: 3.1.1
+        Use `hysteresis` parameter of `set_trigger` instead.
+
         :type strict: `boolean`
         :param strict: Disable all implicit conversions and coercions.
 
@@ -563,9 +635,14 @@ class LockInAmp(Moku):
 
         """
         operation = "set_hysteresis"
-        params = dict(strict=strict, hysteresis_mode=validate_range(
-            hysteresis_mode, ['Absolute', 'Relative']), value=value, )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            hysteresis_mode=hysteresis_mode,
+            value=value,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def enable_rollmode(self, roll=True, strict=True):
         """
@@ -583,9 +660,13 @@ class LockInAmp(Moku):
             strict=strict,
             roll=roll,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_data(self, timeout=60, wait_reacquire=False):
+    def get_data(
+        self, timeout=60, wait_reacquire=False, wait_complete=False, measurements=False
+    ):
         """
         get_data.
 
@@ -595,6 +676,11 @@ class LockInAmp(Moku):
         :type wait_reacquire: `boolean`
         :param wait_reacquire: Wait until new dataframe is reacquired
 
+        :type wait_complete: `boolean`
+        :param wait_complete: Wait until entire frame is available
+
+        :type measurements: `boolean`
+        :param measurements: When True, includes available measurements for each channel
 
         .. important::
             Default timeout for reading the data is 10 seconds. It
@@ -608,29 +694,29 @@ class LockInAmp(Moku):
         params = dict(
             timeout=timeout,
             wait_reacquire=wait_reacquire,
+            wait_complete=wait_complete,
+            measurements=measurements,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
-    def save_high_res_buffer(self, comments=""):
+    def save_high_res_buffer(self, comments="", timeout=60):
         """
         save_high_res_buffer.
 
         :type comments: `string`
         :param comments: Optional comments to be included
 
+        :type timeout: `int`
+        :param timeout: Wait for n seconds before trigger event happens to save the buffer # noqa
+
         """
         operation = "save_high_res_buffer"
-        params = dict(
-            comments=comments,
+        params = dict(comments=comments, timeout=timeout)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
-
-    def get_samplerate(self):
-        """
-        get_samplerate.
-        """
-        operation = "get_samplerate"
-        return self.session.get(self.operation_group, operation)
 
     def set_acquisition_mode(self, mode="Normal", strict=True):
         """
@@ -639,45 +725,56 @@ class LockInAmp(Moku):
         :type strict: `boolean`
         :param strict: Disable all implicit conversions and coercions.
 
-        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
         :param mode: Acquisition Mode
 
         """
         operation = "set_acquisition_mode"
-        params = dict(strict=strict, mode=validate_range(
-            mode, ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']), )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(strict=strict, mode=mode)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+
+    def get_samplerate(self):
+        """
+        get_samplerate.
+        """
+        operation = "get_samplerate"
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def get_acquisition_mode(self):
         """
         get_acquisition_mode.
         """
         operation = "get_acquisition_mode"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def get_timebase(self):
         """
         get_timebase.
         """
         operation = "get_timebase"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def logging_progress(self):
         """
         logging_progress.
         """
         operation = "logging_progress"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def start_logging(
-            self,
-            duration=60,
-            delay=0,
-            file_name_prefix="",
-            comments="",
-            mode="Normal",
-            rate=1000,
-            strict=True):
+        self,
+        duration=60,
+        delay=0,
+        file_name_prefix="",
+        comments="",
+        trigger_source=None,
+        trigger_level=None,
+        mode="Normal",
+        rate=None,
+        strict=True,
+    ):
         """
         start_logging.
 
@@ -690,13 +787,19 @@ class LockInAmp(Moku):
         :type delay: `integer` Sec (defaults to 0)
         :param delay: Delay the start by
 
+        :type trigger_source: `string`  ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External'] # noqa
+        :param trigger_source: Trigger source
+
+        :type trigger_level: `number` [-5V, 5V]  (defaults to 0)
+        :param trigger_level: Trigger level
+
         :type file_name_prefix: `string`
         :param file_name_prefix: Optional file name prefix
 
         :type comments: `string`
         :param comments: Optional comments to be included
 
-        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
         :param mode: Acquisition Mode
 
         :type rate: `number`
@@ -708,16 +811,107 @@ class LockInAmp(Moku):
             strict=strict,
             duration=duration,
             delay=delay,
+            trigger_source=trigger_source,
+            trigger_level=trigger_level,
             file_name_prefix=file_name_prefix,
             comments=comments,
-            mode=validate_range(mode, ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']),
+            mode=mode,
             rate=rate,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def stop_logging(self):
         """
         stop_logging.
         """
         operation = "stop_logging"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
+
+    def start_streaming(
+        self,
+        duration=None,
+        mode="Normal",
+        rate=None,
+        trigger_source=None,
+        trigger_level=None,
+    ):
+        """
+        start_streaming.
+
+        :type duration: `integer`
+        :param duration: Duration in second(s) to stream for
+
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
+        :param mode: Acquisition Mode
+
+        :type rate: `number`
+        :param rate: Acquisition rate
+
+        :type trigger_source: `string`  ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External'] # noqa
+        :param trigger_source: Trigger source
+
+        :type trigger_level: `number` [-5V, 5V]  (defaults to 0)
+        :param trigger_level: Trigger level
+
+
+        """
+        super().start_streaming()
+        operation = "start_streaming"
+        params = dict(
+            duration=duration,
+            mode=mode,
+            rate=rate,
+            trigger_source=trigger_source,
+            trigger_level=trigger_level,
+        )
+
+        response = self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+        self.stream_id = response["stream_id"]
+        self.ip_address = self.session.ip_address
+        return response
+
+    def stop_streaming(self):
+        """
+        stop_streaming.
+
+        """
+        operation = "stop_streaming"
+        response = self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation
+        )
+        self.stream_id = None
+        return response
+
+    def get_chunk(self):
+        """
+        get_chunk.
+
+        Get the next raw chunk from the streaming session
+
+        """
+        data = {"stream_id": {self.stream_id: {"topic": f"logformat{self.slot-1}"}}}
+        result = self.session.post_to_v2_raw("get_chunk", params=data)
+
+        if result.status_code != 200:
+            raise StreamException("Error fetching stream.")
+
+        try:
+            error = json.loads(result.content)
+            raise StreamException(error.get("error", error))
+        except StreamException as e:
+            raise e
+        except Exception:
+            return result.content
+
+    def get_stream_status(self):
+        """
+        get_stream_status.
+
+        """
+        operation = "get_stream_status"
+
+        return self.session.post(f"slot{self.slot}/{self.operation_group}", operation)
