@@ -1,10 +1,11 @@
-from moku import Moku
-from moku.exceptions import MokuException
-from moku.utilities import find_moku_by_serial
-from moku.utilities import validate_range
+import json
+
+from moku import Moku, MultiInstrumentSlottable
+from moku.exceptions import StreamException
+from moku.instruments._stream import StreamInstrument
 
 
-class PIDController(Moku):
+class PIDController(MultiInstrumentSlottable, Moku, StreamInstrument):
     """
     PIDController instrument object.
 
@@ -18,40 +19,77 @@ class PIDController(Moku):
 
     """
 
-    def __init__(self, ip=None, serial=None, force_connect=False,
-                 ignore_busy=False, persist_state=False,
-                 connect_timeout=15, read_timeout=30):
-        self.id = 5
-        self.operation_group = "pidcontroller"
+    INSTRUMENT_ID = 5
+    OPERATION_GROUP = "pidcontroller"
 
-        if not any([ip, serial]):
-            raise MokuException("IP (or) Serial is required")
-        if serial:
-            ip = find_moku_by_serial(serial)
+    def __init__(
+        self,
+        ip=None,
+        serial=None,
+        force_connect=False,
+        ignore_busy=False,
+        persist_state=False,
+        connect_timeout=15,
+        read_timeout=30,
+        slot=None,
+        multi_instrument=None,
+        **kwargs,
+    ):
+        self._init_instrument(
+            ip=ip,
+            serial=serial,
+            force_connect=force_connect,
+            ignore_busy=ignore_busy,
+            persist_state=persist_state,
+            connect_timeout=connect_timeout,
+            read_timeout=read_timeout,
+            slot=slot,
+            multi_instrument=multi_instrument,
+            **kwargs,
+        )
+        StreamInstrument.__init__(self, self.mokuOS_version)
 
-        super().__init__(ip=ip, force_connect=force_connect,
-                         ignore_busy=ignore_busy,
-                         persist_state=persist_state,
-                         connect_timeout=connect_timeout,
-                         read_timeout=read_timeout)
+    @classmethod
+    def for_slot(cls, slot, multi_instrument):
+        """Configures instrument at given slot in multi instrument mode"""
+        return cls(slot=slot, multi_instrument=multi_instrument)
 
-        self.upload_bitstream(self.id)
+    def save_settings(self, filename):
+        """
+        Save instrument settings to a file. The file name should have
+        a `.mokuconf` extension to be compatible with other tools.
+
+        :type filename: FileDescriptorOrPath
+        :param filename: The path to save the `.mokuconf` file to.
+        """
+        self.session.get_file(f"slot{self.slot}/{self.operation_group}", "save_settings", filename)
+    
+    def load_settings(self, filename):
+        """
+        Load a previously saved `.mokuconf` settings file into the instrument.
+        To create a `.mokuconf` file, either use `save_settings` or the desktop app.
+
+        :type filename: FileDescriptorOrPath
+        :param filename: The path to the `.mokuconf` configuration to load
+        """
+        with open(filename, 'rb') as f:
+            self.session.post_file(f"slot{self.slot}/{self.operation_group}", "load_settings", data=f)
 
     def summary(self):
         """
         summary.
         """
         operation = "summary"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def set_defaults(self):
         """
         set_defaults.
         """
         operation = "set_defaults"
-        return self.session.post(self.operation_group, operation)
+        return self.session.post(f"slot{self.slot}/{self.operation_group}", operation)
 
-    def set_frontend(self, channel, impedance, coupling, range, strict=True):
+    def set_frontend(self, channel, impedance, coupling, attenuation=None, gain=None, strict=True):
         """
         set_frontend.
 
@@ -62,32 +100,31 @@ class PIDController(Moku):
         :param channel: Target channel
 
         :type impedance: `string` ['1MOhm', '50Ohm']
-        :param impedance: Impedance
+        :param impedance: Input impedance
 
         :type coupling: `string` ['AC', 'DC']
-        :param coupling: Input Coupling
+        :param coupling: Input coupling
 
-        :type range: `string` ['Default', '400mVpp', '1Vpp', '4Vpp', '10Vpp', '40Vpp', '50Vpp']
-        :param range: Input Range
+        :type attenuation: `string` ['-20dB', '0dB', '14dB', '20dB', '32dB', '40dB'] # noqa
+        :param attenuation: Input attenuation.
 
+        :type gain: `string` ['20dB', '0dB', '-14dB', '-20dB', '-32dB', '-40dB'] # noqa
+        :param attenuation: Input gain.
         """
         operation = "set_frontend"
         params = dict(
-            strict=strict, channel=channel, impedance=validate_range(
-                impedance, [
-                    '1MOhm', '50Ohm']), coupling=validate_range(
-                coupling, [
-                    'AC', 'DC']), range=validate_range(
-                        range, [
-                            'Default', '400mVpp', '1Vpp', '4Vpp', '10Vpp', '40Vpp', '50Vpp']), )
-        return self.session.post(self.operation_group, operation, params)
+            strict=strict,
+            channel=channel,
+            impedance=impedance,
+            coupling=coupling,
+            attenuation=attenuation,
+            gain=gain,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def set_control_matrix(
-            self,
-            channel,
-            input_gain1,
-            input_gain2,
-            strict=True):
+    def set_control_matrix(self, channel, input_gain1, input_gain2, strict=True):
         """
         set_control_matrix.
 
@@ -111,15 +148,11 @@ class PIDController(Moku):
             input_gain1=input_gain1,
             input_gain2=input_gain2,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def enable_output(
-            self,
-            channel,
-            signal,
-            output,
-            enable_gain=False,
-            strict=True):
+    def enable_output(self, channel, signal, output, strict=True):
         """
         enable_output.
 
@@ -135,9 +168,6 @@ class PIDController(Moku):
         :type output: `boolean`
         :param output: Enable output
 
-        :type enable_gain: `boolean`
-        :param enable_gain: If applicable, sets the output gain
-
         """
         operation = "enable_output"
         params = dict(
@@ -145,9 +175,50 @@ class PIDController(Moku):
             channel=channel,
             signal=signal,
             output=output,
-            enable_gain=enable_gain,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+
+    def set_output_gain(self, channel, gain="0dB", strict=True):
+        """
+        set_output_gain.
+
+        :type strict: `boolean`
+        :param strict: Disable all implicit conversions and coercions.
+
+        :type channel: `integer`
+        :param channel: Target channel
+
+        :type gain: `string`
+        :param gain: Set output gain
+
+        """
+        operation = "set_output_gain"
+        params = dict(
+            strict=strict,
+            channel=channel,
+            gain=gain,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+
+    def get_output_gain(self, channel):
+        """
+        get_output_gain.
+
+        :type channel: `integer`
+        :param channel: Target channel
+
+        """
+        operation = "get_output_gain"
+        params = dict(
+            channel=channel,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def enable_input(self, channel, enable, strict=True):
         """
@@ -169,7 +240,9 @@ class PIDController(Moku):
             channel=channel,
             enable=enable,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_input_offset(self, channel, offset, strict=True):
         """
@@ -191,7 +264,9 @@ class PIDController(Moku):
             channel=channel,
             offset=offset,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_output_offset(self, channel, offset, strict=True):
         """
@@ -213,7 +288,9 @@ class PIDController(Moku):
             channel=channel,
             offset=offset,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_monitor(self, monitor_channel, source, strict=True):
         """
@@ -225,40 +302,31 @@ class PIDController(Moku):
         :type monitor_channel: `integer`
         :param monitor_channel: Monitor channel
 
-        :type source: `string` ['None', 'Input1', 'Control1', 'Output1', 'Input2', 'Control2', 'Output2', 'Input3', 'Control3', 'Output3', 'Input4', 'Control4', 'Output4']
-        :param source: Monitor channel source. The source is one of: Input1 : Channel 1 ADC input, Control1 : PID Channel 1 input (after mixing, offset and scaling), Output1 : PID Channel 1 output, Input2 : Channel 2 ADC Input, Control2 : PID Channel 2 input (after mixing, offset and scaling), Output2 : PID Channel 2 output
+        :type source: `string` ['None', 'Input1', 'Control1', 'Output1', 'Input2', 'Control2', 'Output2', 'Input3', 'Control3', 'Output3', 'Input4', 'Control4', 'Output4'] # noqa
+        :param source: Monitor channel source. The source is one of: Input1 : Channel 1 ADC input, Control1 : PID Channel 1 input (after mixing, offset and scaling), Output1 : PID Channel 1 output, Input2 : Channel 2 ADC Input, Control2 : PID Channel 2 input (after mixing, offset and scaling), Output2 : PID Channel 2 output # noqa
 
         """
         operation = "set_monitor"
-        params = dict(strict=strict,
-                      monitor_channel=monitor_channel,
-                      source=validate_range(source,
-                                            ['None',
-                                             'Input1',
-                                             'Control1',
-                                             'Output1',
-                                             'Input2',
-                                             'Control2',
-                                             'Output2',
-                                             'Input3',
-                                             'Control3',
-                                             'Output3',
-                                             'Input4',
-                                             'Control4',
-                                             'Output4']),
-                      )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            monitor_channel=monitor_channel,
+            source=source,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_by_frequency(
-            self,
-            channel,
-            prop_gain=None,
-            int_crossover=None,
-            diff_crossover=None,
-            double_int_crossover=None,
-            int_saturation=None,
-            diff_saturation=None,
-            strict=True):
+        self,
+        channel,
+        prop_gain=None,
+        int_crossover=None,
+        diff_crossover=None,
+        double_int_crossover=None,
+        int_saturation=None,
+        diff_saturation=None,
+        strict=True,
+    ):
         """
         set_by_frequency.
 
@@ -298,19 +366,22 @@ class PIDController(Moku):
             int_saturation=int_saturation,
             diff_saturation=diff_saturation,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_by_gain_and_section(
-            self,
-            channel,
-            section,
-            overall_gain=None,
-            prop_gain=None,
-            int_gain=None,
-            diff_gain=None,
-            int_corner=None,
-            diff_corner=None,
-            strict=True):
+        self,
+        channel,
+        section,
+        overall_gain=None,
+        prop_gain=None,
+        int_gain=None,
+        diff_gain=None,
+        int_corner=None,
+        diff_corner=None,
+        strict=True,
+    ):
         """
         set_by_gain_and_section.
 
@@ -354,18 +425,21 @@ class PIDController(Moku):
             int_corner=int_corner,
             diff_corner=diff_corner,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_by_gain(
-            self,
-            channel,
-            overall_gain=None,
-            prop_gain=None,
-            int_gain=None,
-            diff_gain=None,
-            int_corner=None,
-            diff_corner=None,
-            strict=True):
+        self,
+        channel,
+        overall_gain=None,
+        prop_gain=None,
+        int_gain=None,
+        diff_gain=None,
+        int_corner=None,
+        diff_corner=None,
+        strict=True,
+    ):
         """
         set_by_gain.
 
@@ -405,24 +479,28 @@ class PIDController(Moku):
             int_corner=int_corner,
             diff_corner=diff_corner,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_trigger(
-            self,
-            type="Edge",
-            level=0,
-            mode="Auto",
-            edge="Rising",
-            polarity="Positive",
-            width=0.0001,
-            width_condition="LessThan",
-            nth_event=1,
-            holdoff=0,
-            auto_sensitivity=True,
-            noise_reject=False,
-            hf_reject=False,
-            source="ProbeA",
-            strict=True):
+        self,
+        type="Edge",
+        level=0,
+        mode="Auto",
+        edge="Rising",
+        polarity="Positive",
+        width=0.0001,
+        width_condition="LessThan",
+        nth_event=1,
+        holdoff=0,
+        hysteresis=1e-3,
+        auto_sensitivity=None,
+        noise_reject=False,
+        hf_reject=False,
+        source="ProbeA",
+        strict=True,
+    ):
         """
         set_trigger.
 
@@ -439,7 +517,7 @@ class PIDController(Moku):
         :param mode: Trigger mode
 
         :type edge: `string` ['Rising', 'Falling', 'Both']
-        :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising) or negative (falling), with the 'both' option being invalid
+        :param edge: Which edge to trigger on. In Pulse Width modes this specifies whether the pulse is positive (rising) or negative (falling), with the 'both' option being invalid # noqa
 
         :type polarity: `string` ['Positive', 'Negative']
         :param polarity: Trigger pulse polarity (Pulse mode only)
@@ -451,114 +529,112 @@ class PIDController(Moku):
         :param width_condition: Trigger pulse width condition (pulse mode only)
 
         :type nth_event: `integer` [0, 65535]  (defaults to 1)
-        :param nth_event: The number of trigger events to wait for before triggering
+        :param nth_event: The number of trigger events to wait for before triggering # noqa
 
         :type holdoff: `number` [1e-9Sec, 10Sec]  (defaults to 0)
-        :param holdoff: The duration to hold-off Oscilloscope trigger post trigger event
+        :param holdoff: The duration to hold-off Oscilloscope trigger post trigger event # noqa
+
+        :type hysteresis: `number` (defaults to 0.1)
+        :param hysteresis: Absolute hysteresis around trigger
 
         :type auto_sensitivity: `boolean`
         :param auto_sensitivity: Configure auto or manual hysteresis for noise rejection.
 
         :type noise_reject: `boolean`
-        :param noise_reject: Configure the Oscilloscope with a small amount of hysteresis to prevent repeated triggering due to noise
+        :param noise_reject: Configure the Oscilloscope with a small amount of hysteresis to prevent repeated triggering due to noise # noqa
 
         :type hf_reject: `boolean`
-        :param hf_reject: Configure the trigger signal to pass through a low pass filter to smooths out the noise
+        :param hf_reject: Configure the trigger signal to pass through a low pass filter to smooths out the noise # noqa
 
-        :type source: `string` ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD']
+        :type source: `string` ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External']
         :param source: Trigger Source
 
         """
         operation = "set_trigger"
         params = dict(
             strict=strict,
-            type=validate_range(type, ['Edge', 'Pulse']),
+            type=type,
             level=level,
-            mode=validate_range(mode, ['Auto', 'Normal']),
-            edge=validate_range(edge, ['Rising', 'Falling', 'Both']),
-            polarity=validate_range(polarity, ['Positive', 'Negative']),
+            mode=mode,
+            edge=edge,
+            polarity=polarity,
             width=width,
-            width_condition=validate_range(width_condition, ['GreaterThan', 'LessThan']),
+            width_condition=width_condition,
             nth_event=nth_event,
             holdoff=holdoff,
+            hysteresis=hysteresis,
             auto_sensitivity=auto_sensitivity,
             noise_reject=noise_reject,
             hf_reject=hf_reject,
-            source=validate_range(source, ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD']),
+            source=source,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
-    def get_frontend(self, channel, strict=True):
+    def get_frontend(self, channel):
         """
         get_frontend.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_frontend"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_control_matrix(self, channel, strict=True):
+    def get_control_matrix(self, channel):
         """
         get_control_matrix.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_control_matrix"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_input_offset(self, channel, strict=True):
+    def get_input_offset(self, channel):
         """
         get_input_offset.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_input_offset"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_output_offset(self, channel, strict=True):
+    def get_output_offset(self, channel):
         """
         get_output_offset.
 
-        :type strict: `boolean`
-        :param strict: Disable all implicit conversions and coercions.
-
         :type channel: `integer`
         :param channel: Target channel
 
         """
         operation = "get_output_offset"
         params = dict(
-            strict=strict,
             channel=channel,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_timebase(self, t1, t2, strict=True):
         """
@@ -568,10 +644,10 @@ class PIDController(Moku):
         :param strict: Disable all implicit conversions and coercions.
 
         :type t1: `number`
-        :param t1: Time from the trigger point to the left of screen. This may be negative (trigger on-screen) or positive (trigger off the left of screen).
+        :param t1: Time from the trigger point to the left of screen. This may be negative (trigger on-screen) or positive (trigger off the left of screen). # noqa
 
         :type t2: `number`
-        :param t2: Time from the trigger point to the right of screen. (Must be a positive number, i.e. after the trigger event)
+        :param t2: Time from the trigger point to the right of screen. (Must be a positive number, i.e. after the trigger event) # noqa
 
         """
         operation = "set_timebase"
@@ -580,12 +656,17 @@ class PIDController(Moku):
             t1=t1,
             t2=t2,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def set_hysteresis(self, hysteresis_mode, value=0, strict=True):
         """
         set_hysteresis.
 
+        .. deprecated:: 3.1.1
+        Use `hysteresis` parameter of `set_trigger` instead.
+
         :type strict: `boolean`
         :param strict: Disable all implicit conversions and coercions.
 
@@ -597,9 +678,14 @@ class PIDController(Moku):
 
         """
         operation = "set_hysteresis"
-        params = dict(strict=strict, hysteresis_mode=validate_range(
-            hysteresis_mode, ['Absolute', 'Relative']), value=value, )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(
+            strict=strict,
+            hysteresis_mode=hysteresis_mode,
+            value=value,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def enable_rollmode(self, roll=True, strict=True):
         """
@@ -617,9 +703,13 @@ class PIDController(Moku):
             strict=strict,
             roll=roll,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
-    def get_data(self, timeout=60, wait_reacquire=False):
+    def get_data(
+        self, timeout=60, wait_reacquire=False, wait_complete=False, measurements=False
+    ):
         """
         get_data.
 
@@ -629,6 +719,11 @@ class PIDController(Moku):
         :type wait_reacquire: `boolean`
         :param wait_reacquire: Wait until new dataframe is reacquired
 
+        :type wait_complete: `boolean`
+        :param wait_complete: Wait until entire frame is available
+
+        :type measurements: `boolean`
+        :param measurements: When True, includes available measurements for each channel
 
         .. important::
             Default timeout for reading the data is 10 seconds. It
@@ -642,29 +737,29 @@ class PIDController(Moku):
         params = dict(
             timeout=timeout,
             wait_reacquire=wait_reacquire,
+            wait_complete=wait_complete,
+            measurements=measurements,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
 
-    def save_high_res_buffer(self, comments=""):
+    def save_high_res_buffer(self, comments="", timeout=60):
         """
         save_high_res_buffer.
 
         :type comments: `string`
         :param comments: Optional comments to be included
 
+        :type timeout: `int`
+        :param timeout: Wait for n seconds before trigger event happens to save the buffer # noqa
+
         """
         operation = "save_high_res_buffer"
-        params = dict(
-            comments=comments,
+        params = dict(comments=comments, timeout=timeout)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
         )
-        return self.session.post(self.operation_group, operation, params)
-
-    def get_samplerate(self):
-        """
-        get_samplerate.
-        """
-        operation = "get_samplerate"
-        return self.session.get(self.operation_group, operation)
 
     def set_acquisition_mode(self, mode="Normal", strict=True):
         """
@@ -673,45 +768,110 @@ class PIDController(Moku):
         :type strict: `boolean`
         :param strict: Disable all implicit conversions and coercions.
 
-        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
         :param mode: Acquisition Mode
 
         """
         operation = "set_acquisition_mode"
-        params = dict(strict=strict, mode=validate_range(
-            mode, ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']), )
-        return self.session.post(self.operation_group, operation, params)
+        params = dict(strict=strict, mode=mode)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+
+    def set_output_limit(
+        self, channel, enable=False, low_limit=None, high_limit=None, strict=True
+    ):
+        """
+        set_output_limit.
+
+        :type strict: `boolean`
+        :param strict: Disable all implicit conversions and coercions.
+
+        :type channel: `integer`
+        :param channel: Target channel
+
+        :type enable: `boolean`
+        :param enable: Enable voltage limiter
+
+        :type low_limit: `number`
+        :param low_limit: Low voltage limit
+
+        :type high_limit: `number`
+        :param high_limit: High voltage limit
+
+        """
+        operation = "set_output_limit"
+        params = dict(
+            strict=strict,
+            channel=channel,
+            enable=enable,
+            low_limit=low_limit,
+            high_limit=high_limit,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+
+    def get_output_limit(self, channel, strict=True):
+        """
+        get_output_limit.
+
+        :type strict: `boolean`
+        :param strict: Disable all implicit conversions and coercions.
+
+        :type channel: `integer`
+        :param channel: Target channel
+
+        """
+        operation = "get_output_limit"
+        params = dict(
+            strict=strict,
+            channel=channel,
+        )
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+
+    def get_samplerate(self):
+        """
+        get_samplerate.
+        """
+        operation = "get_samplerate"
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def get_acquisition_mode(self):
         """
         get_acquisition_mode.
         """
         operation = "get_acquisition_mode"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def get_timebase(self):
         """
         get_timebase.
         """
         operation = "get_timebase"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def logging_progress(self):
         """
         logging_progress.
         """
         operation = "logging_progress"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
 
     def start_logging(
-            self,
-            duration=60,
-            delay=0,
-            file_name_prefix="",
-            comments="",
-            mode="Normal",
-            rate=1000,
-            strict=True):
+        self,
+        duration=60,
+        delay=0,
+        file_name_prefix="",
+        comments="",
+        trigger_source=None,
+        trigger_level=None,
+        mode="Normal",
+        rate=None,
+        strict=True,
+    ):
         """
         start_logging.
 
@@ -724,13 +884,19 @@ class PIDController(Moku):
         :type delay: `integer` Sec (defaults to 0)
         :param delay: Delay the start by
 
+        :type trigger_source: `string`  ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External'] # noqa
+        :param trigger_source: Trigger source
+
+        :type trigger_level: `number` [-5V, 5V]  (defaults to 0)
+        :param trigger_level: Trigger level
+
         :type file_name_prefix: `string`
         :param file_name_prefix: Optional file name prefix
 
         :type comments: `string`
         :param comments: Optional comments to be included
 
-        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
         :param mode: Acquisition Mode
 
         :type rate: `number`
@@ -742,16 +908,106 @@ class PIDController(Moku):
             strict=strict,
             duration=duration,
             delay=delay,
+            trigger_source=trigger_source,
+            trigger_level=trigger_level,
             file_name_prefix=file_name_prefix,
             comments=comments,
-            mode=validate_range(mode, ['Normal', 'Precision', 'DeepMemory', 'PeakDetect']),
+            mode=mode,
             rate=rate,
         )
-        return self.session.post(self.operation_group, operation, params)
+        return self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
 
     def stop_logging(self):
         """
         stop_logging.
         """
         operation = "stop_logging"
-        return self.session.get(self.operation_group, operation)
+        return self.session.get(f"slot{self.slot}/{self.operation_group}", operation)
+
+    def start_streaming(
+        self,
+        duration=None,
+        mode="Normal",
+        rate=None,
+        trigger_source=None,
+        trigger_level=None,
+    ):
+        """
+        start_streaming.
+
+        :type duration: `integer`
+        :param duration: Duration in second(s) to stream for
+
+        :type mode: `string` ['Normal', 'Precision', 'DeepMemory', 'PeakDetect'] # noqa
+        :param mode: Acquisition Mode
+
+        :type rate: `number`
+        :param rate: Acquisition rate
+
+        :type trigger_source: `string`  ['ProbeA', 'ProbeB', 'ProbeC', 'ProbeD', 'External'] # noqa
+        :param trigger_source: Trigger source
+
+        :type trigger_level: `number` [-5V, 5V]  (defaults to 0)
+        :param trigger_level: Trigger level
+
+
+        """
+        super().start_streaming()
+        operation = "start_streaming"
+        params = dict(
+            duration=duration,
+            mode=mode,
+            rate=rate,
+            trigger_source=trigger_source,
+            trigger_level=trigger_level,
+        )
+        response = self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation, params
+        )
+        self.stream_id = response["stream_id"]
+        self.ip_address = self.session.ip_address
+        return response
+
+    def stop_streaming(self):
+        """
+        stop_streaming.
+
+        """
+        operation = "stop_streaming"
+        response = self.session.post(
+            f"slot{self.slot}/{self.operation_group}", operation
+        )
+        self.stream_id = None
+        return response
+
+    def get_chunk(self):
+        """
+        get_chunk.
+
+        Get the next raw chunk from the streaming session
+
+        """
+        data = {"stream_id": {self.stream_id: {"topic": f"logformat{self.slot-1}"}}}
+        result = self.session.post_to_v2_raw("get_chunk", params=data)
+
+        if result.status_code != 200:
+            raise StreamException("Error fetching stream.")
+
+        try:
+            error = json.loads(result.content)
+            raise StreamException(error.get("error", error))
+        except StreamException as e:
+            raise e
+        except Exception:
+            return result.content
+
+    def get_stream_status(self):
+        """
+        get_stream_status.
+
+        """
+        operation = "get_stream_status"
+
+        return self.session.post(f"slot{self.slot}/{self.operation_group}", operation)
